@@ -5,8 +5,8 @@ use strict;
 
 use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION);
 
-$VERSION = '3.29_01';
-$VERSION = eval $VERSION;
+$VERSION = '3.46_01';
+$VERSION =~ tr/_//;
 
 require Exporter;
 
@@ -37,10 +37,30 @@ require Exporter;
 
 %EXPORT_TAGS = ( ALL => [ @EXPORT_OK, @EXPORT ] );
 
+require File::Spec::Unix;
+my %udeps = (
+    canonpath => [],
+    catdir => [qw(canonpath)],
+    catfile => [qw(canonpath catdir)],
+    case_tolerant => [],
+    curdir => [],
+    devnull => [],
+    rootdir => [],
+    updir => [],
+);
+
 foreach my $meth (@EXPORT, @EXPORT_OK) {
     my $sub = File::Spec->can($meth);
     no strict 'refs';
-    *{$meth} = sub {&$sub('File::Spec', @_)};
+    if (exists($udeps{$meth}) && $sub == File::Spec::Unix->can($meth) &&
+	    !(grep {
+		File::Spec->can($_) != File::Spec::Unix->can($_)
+	    } @{$udeps{$meth}}) &&
+	    defined(&{"File::Spec::Unix::_fn_$meth"})) {
+	*{$meth} = \&{"File::Spec::Unix::_fn_$meth"};
+    } else {
+	*{$meth} = sub {&$sub('File::Spec', @_)};
+    }
 }
 
 
